***********************************************************************
* This file contains the workspace definition for the current
* application. If the workspace is not large enough (not enough
* memory), increase the NNWORK parameter here and recompile
* the application!
***********************************************************************

      PARAMETER (NNWORK=8000000)
